
/************************************************************
*                                                           *
* AlbumFormatStorer.java                                    *
* Part of the CDDB-Mp3-Renamer source code licensed under   *
*   the GNU GENERAL PUBLIC LICENSE (GPL).                   *
* Author: Mikael Neman                                      *
* Date: 2003-07-23                                          *
* HTTP: http://cddb-mp3-renamer.flashdance.cx               *
* Description: A class for storing album information.       *
*              Also used to clean up junk characters.       *
*              This class can be expanded to hold different *
*              album formats if needed.                     *
*                                                           *
************************************************************/

class AlbumFormatStorer implements Constants
{
  AlbumFormatStorer() {}

  private String [] normalArtistModeStrings;
  private String [] noArtistModeStrings;

  void setNormalArtistModeStrings(String [] s)
  {
    normalArtistModeStrings=new String[s.length];
    for(int q=0; q < s.length; q++)
      normalArtistModeStrings[q] = new String(s[q]);

    removeJunkChars(normalArtistModeStrings);
  }

  void setNoArtistModeStrings(String [] s)
  {
    noArtistModeStrings=new String[s.length];
    for(int q=0; q < s.length; q++)
      noArtistModeStrings[q] = new String(s[q]);

    removeJunkChars(noArtistModeStrings);
  }

  String [] getNormalArtistModeString()
  {
    return (normalArtistModeStrings);
  }

  String [] getNoArtistModeString()
  {
    return (noArtistModeStrings);
  }

  void removeJunkChars(String [] s)
  {
    //Need to parse all strings and remove all 'junk' in them here
    //Starting with string replacing
    for(int q=0; q < s.length; q++)
    {
      s[q] = replaceString(s[q], "&#034;", "\"");
      s[q] = replaceString(s[q], "&#34;", "\"");
      s[q] = replaceString(s[q], "&#039;", "'");
      s[q] = replaceString(s[q], "&#39;", "'");
      s[q] = replaceString(s[q], "&#035;", "#");
      s[q] = replaceString(s[q], "&#35;", "#");
      s[q] = replaceString(s[q], "&#038;", "&");
      s[q] = replaceString(s[q], "&#38;", "&");

      //Illegal filename chars (in windows)
      s[q] = replaceString(s[q], "/", "-");
      s[q] = replaceString(s[q], "\\", "");
      s[q] = replaceString(s[q], ":", "");
      s[q] = replaceString(s[q], "*", "");
      s[q] = replaceString(s[q], "?", "");
      s[q] = replaceString(s[q], "\"", "");
      s[q] = replaceString(s[q], "<", "");
      s[q] = replaceString(s[q], ">", "");
      s[q] = replaceString(s[q], "|", "");

      //
      s[q] = replaceString(s[q], "å", "");
      s[q] = replaceString(s[q], "ä", "");
      s[q] = replaceString(s[q], "ö", "");
      s[q] = replaceString(s[q], "Å", "");
      s[q] = replaceString(s[q], "Ä", "");
      s[q] = replaceString(s[q], "Ö", "");

    }
  }

  String replaceString(String inputString, String stringToBeRemoved, String stringToReplaceWith)
  {
    if(DEBUG_MODE)
      System.out.println("inputString: " + inputString + "\n removeString: " + stringToBeRemoved +
        " replaceWith: " + stringToReplaceWith);
    int endIndex = inputString.indexOf(stringToBeRemoved);
    if(endIndex == -1) //Nothing to be done
    {
      return(inputString);
    }

    //Time to parse this shit for real ;)
    StringBuffer preString = new StringBuffer(inputString.length());
    int startIndex = 0;
    int searchFromHere;

    while(true)
    {
      //Copy original string to first occurance of stringToBeRemoved
      preString.append(inputString.substring(startIndex, endIndex));
      //Insert the replacement string
      preString.append(stringToReplaceWith);
      startIndex = endIndex + stringToBeRemoved.length();
      //Search for more occurances
      searchFromHere = startIndex;
      searchFromHere = inputString.indexOf(stringToBeRemoved, searchFromHere);

      if(searchFromHere == -1) //I'm done now, add the last part and exit
      {
        preString.append(inputString.substring(startIndex));
        String newString = new String(preString.toString());
        if(DEBUG_MODE)
          System.out.println("inputString = " + inputString + "\n newString = " + newString);
        return(newString);
      }
      else
        endIndex = searchFromHere;
    }
  }
}