
/************************************************************
*                                                           *
* CDDBRenamer.java                                          *
* Part of the CDDB-Mp3-Renamer source code licensed under   *
*   the GNU GENERAL PUBLIC LICENSE (GPL).                   *
* Author: Mikael Neman                                      *
* Date: 2003-07-23                                          *
* HTTP: http://cddb-mp3-renamer.flashdance.cx               *
* Description: Main class of the program. Contains the      *
*              GUI parts of the program and most of the     *
*              logic.                                       *
*                                                           *
************************************************************/

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

import de.ueberdosis.mp3info.*;
import de.ueberdosis.mp3info.id3v2.*;

class CDDBRenamer extends JFrame implements Constants, ListSelectionListener, ActionListener
{
  private JList albumList;
  private JList songList;
  private JList localSongList;
  private String [] localFilesStrings;
  private File [] localFiles;
  private File [] tempLocalFiles; //Needs to be global because of threads
  private int nrOfFilesInDirectory; //Needs to be global because of threads
  private JSplitPane splitPane;
  private JScrollPane localSongListScrollPane;
  private JPanel leftSide;
  private JPanel rightSide;
  private JPanel topSide;
  JPanel downSide;
  private JButton openButton;
  private JButton searchButton;
  private JButton renameButton;
  private JTextField searchTextField;
  private String finalSearchString;   //Needs to be global because of threads
  private AlbumFormatStorer allSongsOfThisAlbum;  //Needs to be global because of threads
  private JFileChooser fileChooser = new JFileChooser();
  private boolean remoteAlbumIsSelected = false;
  private boolean localAlbumIsSelected = false;
  JLabel statusBar;
  HTTPParser hp=new HTTPParser(statusBar, downSide);
  int indexOfPages; //No of entries in hls
  HTMLLinkStorer [] hls; //Maximum no of enries is 200
  private ID3Tag [] id3Tags; //the tags to store the information
  private ID3V2Tag [] id3v2Tags; //v2! whoooo

  public CDDBRenamer()
  {
    super();
    //This is where to put the current version information.
    String programTitle = new String("CDDB Mp3 Renamer v1.0.10");
    if(DEBUG_MODE)
      programTitle = new String(programTitle + "     DEBUG VERSION, DO NOT DISTRIBUTE!");
    this.setTitle(programTitle);
    indexOfPages=0;
    hls = new HTMLLinkStorer[200]; //Maximum no of entries is 200

    //Swing stuff
    leftSide = new JPanel(new BorderLayout());
    rightSide = new JPanel(new BorderLayout());
    topSide = new JPanel(new BorderLayout(15,2));
    downSide = new JPanel(new BorderLayout());

    localFilesStrings = new String[2];
    localFilesStrings[0] = new String("Open the directory or a file in the directory ");
    localFilesStrings[1] = new String("you want to rename files in.");

    //ID3
    id3Tags = new ID3Tag[100];
    id3v2Tags = new ID3V2Tag[100];
    //END ID3

    localSongList = new JList(localFilesStrings);
    localSongList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    openButton = new JButton("Open File");
    openButton.setActionCommand("open");
    openButton.addActionListener(this);

    localSongListScrollPane = new JScrollPane(localSongList);
    //Add stuff to the left panel
    leftSide.add(localSongListScrollPane, BorderLayout.CENTER);
    leftSide.add(openButton, BorderLayout.NORTH);

    albumList = new JList(); //This should take the list of album names later
    albumList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    albumList.setSelectedIndex(0);
    albumList.addListSelectionListener(this);
    JScrollPane albumListScrollPane = new JScrollPane(albumList);

    songList = new JList(); //This should take the list of song names later
    JScrollPane songListScrollPane = new JScrollPane(songList);

    //Create a split pane with the two scroll panes in it
    splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, albumListScrollPane, songListScrollPane);
    splitPane.setOneTouchExpandable(true);
    splitPane.setDividerLocation(200);

/*
	Provide minimum sizes for the two components in the split pane
    Dimension minimumSize = new Dimension(100, 150);
    albumListScrollPane.setMinimumSize(minimumSize);
    songListScrollPane.setMinimumSize(minimumSize);
*/
    //Provide a preferred size for the split pane
    splitPane.setPreferredSize(new Dimension(500, 400));

    addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { System.exit(0); } } );
    rightSide.add(getSplitPane(),BorderLayout.CENTER);

    //Adding stuff to top side
    searchTextField = new JTextField(25);
    searchTextField.setActionCommand("SearchTextFieldString");

    searchButton = new JButton("Search!");
    searchButton.setPreferredSize(new Dimension(150,5));
    renameButton = new JButton("Rename Files");
    renameButton.setPreferredSize(new Dimension(200,5));
    renameButton.setActionCommand("rename");
    renameButton.addActionListener(this);
    renameButton.setEnabled(false);
    searchButton.setActionCommand("search");
    searchButton.addActionListener(this);

    topSide.add(searchTextField, BorderLayout.WEST);
    topSide.add(searchButton, BorderLayout.CENTER);
    topSide.add(renameButton, BorderLayout.EAST);

    statusBar = new JLabel("Waiting for input from user...");
    downSide.add(statusBar, BorderLayout.CENTER);

    //Adding the panels
    getContentPane().add(topSide, BorderLayout.NORTH);
    getContentPane().add(leftSide, BorderLayout.WEST);
    getContentPane().add(rightSide, BorderLayout.CENTER);
    getContentPane().add(downSide, BorderLayout.SOUTH);

    pack();
    setVisible(true);
  }
  public static void main(String[] args) throws IOException
  {
    new CDDBRenamer();
  }
  public void valueChanged(ListSelectionEvent e)
  {
      if (e.getValueIsAdjusting())
          return;

      JList theList = (JList)e.getSource();

      if (theList.isSelectionEmpty())
      {
          songList = new JList();
          JScrollPane songListScrollPane = new JScrollPane(songList);
      } else
      {
          //Time to download the song list from the album
          final int index = theList.getSelectedIndex();
          statusBar.setText("Downloading songs of this album, please wait...");
          //Thread here
          Runnable songDownloadThread = new Runnable()
          {
            public void run()
            {
              allSongsOfThisAlbum = hp.parseAlbum(getHLS(index), id3Tags, id3v2Tags);
              statusBar.setText( (allSongsOfThisAlbum.getNormalArtistModeString().length + 1) + " songs in the album.");
              redrawSplitPane(allSongsOfThisAlbum.getNormalArtistModeString());
              remoteAlbumIsSelected = true;
              if(localAlbumIsSelected)
                renameButton.setEnabled(true);
            }
          };
          new Thread(songDownloadThread).start();
      }
  }
  public JSplitPane getSplitPane()
  {
        return splitPane;
  }
  //Returns an index of HTMLLinkStorer (HLS)
  public HTMLLinkStorer getHLS(int index)
  {
        return hls[index];
  }
  public void actionPerformed(ActionEvent e)
  {
    if(DEBUG_MODE)
      System.out.println("Entering actionPerformed");
    if (e.getActionCommand().equals("search"))
    {
      searchButton.setEnabled(false);
      if(DEBUG_MODE)
        System.out.println("Entering actionPerformed -> search");

      StringBuffer tempSB = new StringBuffer(100);
      String tempString = new String(searchTextField.getText());
      if(tempString.length() == 0)
      {
        statusBar.setText("Error! Type something in the textfield stupid! ;-)");
        searchButton.setEnabled(true);
        return;
      }
      //First, replace all spaces with %20
      for(int i=0; i < tempString.length(); i++)
      {
        if(tempString.charAt(i) == ' ')
          tempSB.append("%20");
        else
          tempSB.append(tempString.charAt(i));
      }

      finalSearchString = new String(tempSB.toString());

      //Reset some values from previous search
      indexOfPages = 0;
      hp = new HTTPParser(statusBar, downSide);

      //Locate possible hits
      //Run in a thread
      Runnable searchThread = new Runnable()
      {
        public void run()
        {
          indexOfPages = hp.locateLinks(hls, indexOfPages, CDDB_HTTP_ADDRESS + finalSearchString);
          //Error, abort, no albums found on the page.
          if(indexOfPages == -1)
          {
            searchButton.setEnabled(true);
            return;
          }
          //Create a list of strings to display
          String [] dispAlbums = new String[indexOfPages];
          for(int q=0; q < indexOfPages; q++)
            dispAlbums[q] = new String(hls[q].getName());

          //If no album was found, hls[0] will be NULL, the program should jump out of this method

          if (hls[0] == null)
          {
            searchButton.setEnabled(true);
            if(DEBUG_MODE)
              System.out.println("No albums were found...");

            return;
          }
          allSongsOfThisAlbum = hp.parseAlbum(hls[0],id3Tags,id3v2Tags);

          if(localAlbumIsSelected)
            renameButton.setEnabled(true);
          searchButton.setEnabled(true);

          //Remove junk characters from the album list
          for(int q=0; q < indexOfPages; q++)
          {
            dispAlbums[q] = replaceString(dispAlbums[q], "&#034;", "\"");
            dispAlbums[q] = replaceString(dispAlbums[q], "&#34;", "\"");
            dispAlbums[q] = replaceString(dispAlbums[q], "&#039;", "'");
            dispAlbums[q] = replaceString(dispAlbums[q], "&#39;", "'");
            dispAlbums[q] = replaceString(dispAlbums[q], "&#035;", "#");
            dispAlbums[q] = replaceString(dispAlbums[q], "&#35;", "#");
            dispAlbums[q] = replaceString(dispAlbums[q], "&#038;", "&");
            dispAlbums[q] = replaceString(dispAlbums[q], "&#38;", "&");
          }

          if(allSongsOfThisAlbum != null)
            redrawSplitPane(dispAlbums, allSongsOfThisAlbum.getNormalArtistModeString());
          else
            statusBar.setText("Error: no Artist information was found on page");
        }
      };

      new Thread(searchThread).start();
    }
    else if(e.getActionCommand().equals("open"))
    {
      openButton.setEnabled(false);
      if(DEBUG_MODE)
        System.out.println("Entering actionPerformed -> open");

      int returnedFile = fileChooser.showOpenDialog(CDDBRenamer.this);
      if (returnedFile == JFileChooser.APPROVE_OPTION)
      {
          File theFile = fileChooser.getSelectedFile();
          //Get the file's directory
          theFile = theFile.getParentFile();
          //List all files as strings, use .mp3 as filter
          Mp3FileFilter mp3Filter = new Mp3FileFilter();
          theFile.listFiles(mp3Filter);

          if(DEBUG_MODE)
            System.out.println("The directory is: '" + theFile.getName() +
            "' Number of listed files =" + theFile.list(mp3Filter).length);

          nrOfFilesInDirectory = theFile.list(mp3Filter).length;
          //In case of no files in directory
          if(nrOfFilesInDirectory == 0)
          {
            statusBar.setText("No mp3 files found in the directory");
            openButton.setEnabled(true);
            return;
          }

          localFilesStrings = new String[nrOfFilesInDirectory];
          localFilesStrings = theFile.list(mp3Filter);

          //Sorting strings
          ArrayList sortShit = new ArrayList();
          for (int q=0; q < nrOfFilesInDirectory; q++)
            sortShit.add(localFilesStrings[q]);

          //Collator skit h?r kanske
          Collections.sort(sortShit);
          localFilesStrings = (String[])sortShit.toArray(localFilesStrings);

          //List all files as Files
          //Need to sort these in the same order as the strings
          localFiles = new File[nrOfFilesInDirectory];
          tempLocalFiles = new File[nrOfFilesInDirectory];
          tempLocalFiles = theFile.listFiles(mp3Filter);

          //Optimize performance
          Runnable sortFilesThread = new Runnable()
          {
            public void run()
            {
              int l;
              int nrOfFilesLeft = nrOfFilesInDirectory;
              for(l=0; l < nrOfFilesInDirectory; l++)
              {
                for(int li=0; li < nrOfFilesLeft; li++)
                {
                  if(localFilesStrings[l].equals(tempLocalFiles[li].getName()))
                  {
                    localFiles[l] = tempLocalFiles[li];
                    tempLocalFiles[li] = tempLocalFiles[nrOfFilesLeft-1];
                    tempLocalFiles[nrOfFilesLeft-1] = null;
                    nrOfFilesLeft--;
                    statusBar.setText("Sorting files... " + (l+1)+ " files sorted of " + nrOfFilesInDirectory + " files in total");
                    break;
                  }
                }
              }
              if(DEBUG_MODE)
                for(int bopp=0; bopp < nrOfFilesInDirectory; bopp++)
                  System.out.println("First name if the file " +localFiles[bopp].getName());

              statusBar.setText("Sorting completed " + (l) + " mp3 files found and sorted in alphabetical order.");
            }
          };
          new Thread(sortFilesThread).start();

          //Remove old
          leftSide.remove(localSongListScrollPane);

          localSongList = new JList(localFilesStrings);
          localSongList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
          localSongListScrollPane = new JScrollPane(localSongList);

          //Add stuff to the left panel
          leftSide.add(localSongListScrollPane, BorderLayout.WEST);
          pack();
          localAlbumIsSelected = true;
          if(remoteAlbumIsSelected)
            renameButton.setEnabled(true);

          openButton.setEnabled(true);
      }

      openButton.setEnabled(true);
    }
    else if(e.getActionCommand().equals("rename"))
    {
      try
      {
        renameFiles();
      }
      catch( Exception ex )
      {
        System.out.println("caught exception: "+ex.getMessage());
      }
    }
  }

  private void renameFiles() throws IOException, FileNotFoundException
  {
       if(DEBUG_MODE)
        System.out.println("Entering actionPerformed -> rename");
      renameButton.setEnabled(false);

/*
      TODO
      Remove NORMAL_ARTIST_MODE and NO_ARTIST_MODE from httpparser etc...
      Add a popup dialog here.
      In the dialog it should be possible to change what album format to use, for example
      Artist - Song name.mp3 or Artist - Album - Songname.mp3 etc...
      The result of the dialog should be returned to "songsToRenameWith"
      When the dialog is open, the main windows should be hidden or blocked.
*/

      int songsOnTheInternet = allSongsOfThisAlbum.getNormalArtistModeString().length;
      String [] songsToRenameWith = new String[songsOnTheInternet];
      songsToRenameWith = allSongsOfThisAlbum.getNormalArtistModeString();
      int songsOnLocalDisk = localFiles.length;
      int i;
      String trackNumber;
      for(i=0; (i < songsOnTheInternet) && (i < songsOnLocalDisk); i++)
      {
        if((i+1) <= 9)
          trackNumber = new String("0" + (i+1) + " - ");
        else
          trackNumber = new String((i+1) + " - ");

        File f = new File(localFiles[i].getParent() + localFiles[i].separatorChar + trackNumber + songsToRenameWith[i] + ".mp3");
        if(DEBUG_MODE)
          System.out.println(i + " f.getAbsolutePath() is: " + f.getAbsolutePath());

        if(!FAKE_RENAME_ONLY)
        {
          if(!localFiles[i].renameTo(f))
            System.out.println("Error trying to rename the file '" + localFiles[i].getName() + "'");

          //ID3
          RandomAccessFile mp3file = new RandomAccessFile(f, "rw");
          ID3Writer.writeTag(mp3file, id3Tags[i]);

          //v2
          ID3V2Writer.writeTag(f, id3v2Tags[i]);

          mp3file.close();
          //END ID3


        }
      }
      if(DEBUG_MODE)
        System.out.println("Ready. ");
      statusBar.setText(i + " files were renamed...");
  }

  //Redraws both fields in the SplitPane
  void redrawSplitPane(String [] albums, String [] songs)
  {
    //Remove the old SplitPane
    rightSide.remove(getSplitPane());

    albumList = new JList(albums); //this should take the list of album names later
    albumList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    albumList.setSelectedIndex(0);
    albumList.addListSelectionListener(this);
    JScrollPane albumListScrollPane = new JScrollPane(albumList);

    songList = new JList(songs); //this should take the list of song names later
    JScrollPane songListScrollPane = new JScrollPane(songList);

    //Create a split pane with the two scroll panes in it
    splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, albumListScrollPane, songListScrollPane);
    splitPane.setOneTouchExpandable(true);
    if(splitPane.getDividerLocation() < 50)
      splitPane.setDividerLocation(250);

/*
    Provide minimum sizes for the two components in the split pane
    Dimension minimumSize = new Dimension(100, 150);
    albumListScrollPane.setMinimumSize(minimumSize);
    songListScrollPane.setMinimumSize(minimumSize);
*/

    //Provide a preferred size for the split pane
    splitPane.setPreferredSize(new Dimension(600, 400));

    addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { System.exit(0); } } );
    rightSide.add(getSplitPane(),BorderLayout.CENTER);
    pack();
    setVisible(true);
  }
  //Redraws only songs field in the SplitPane
  void redrawSplitPane(String [] songs)
  {
    //Remove the old SplitPane
    rightSide.remove(getSplitPane());

    JScrollPane albumListScrollPane = new JScrollPane(albumList);

    songList = new JList(songs); //this should take the list of song names later
    JScrollPane songListScrollPane = new JScrollPane(songList);

    //Create a split pane with the two scroll panes in it
    splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, albumListScrollPane, songListScrollPane);
    splitPane.setOneTouchExpandable(true);
    if(splitPane.getDividerLocation() < 50)
      splitPane.setDividerLocation(250);

/*
    Provide minimum sizes for the two components in the split pane
    Dimension minimumSize = new Dimension(100, 150);
    songListScrollPane.setMinimumSize(minimumSize);
*/

    //Provide a preferred size for the split pane
    splitPane.setPreferredSize(new Dimension(600, 400));

    addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { System.exit(0); } } );
    rightSide.add(getSplitPane(),BorderLayout.CENTER);
    pack();
    setVisible(true);
  }

  String replaceString(String inputString, String stringToBeRemoved, String stringToReplaceWith)
  {
    if(DEBUG_MODE)
      System.out.println("inputString: " + inputString + "\n removeString: " + stringToBeRemoved +
        " replaceWith: " + stringToReplaceWith);
    int endIndex = inputString.indexOf(stringToBeRemoved);
    if(endIndex == -1) //Nothing to be done
    {
      return(inputString);
    }

    //Time to parse this shit for real ;)
    StringBuffer preString = new StringBuffer(inputString.length());
    int startIndex = 0;
    int searchFromHere;

    while(true)
    {
      //Copy original string to first occurance of stringToBeRemoved
      preString.append(inputString.substring(startIndex, endIndex));
      //Insert the replacement string
      preString.append(stringToReplaceWith);
      startIndex = endIndex + stringToBeRemoved.length();
      //Search for more occurances
      searchFromHere = startIndex;
      searchFromHere = inputString.indexOf(stringToBeRemoved, searchFromHere);

      if(searchFromHere == -1) //I'm done now, add the last part and exit
      {
        preString.append(inputString.substring(startIndex));
        String newString = new String(preString.toString());
        if(DEBUG_MODE)
          System.out.println("inputString = " + inputString + "\n newString = " + newString);
        return(newString);
      }
      else
        endIndex = searchFromHere;
    }
  }
}
