
/************************************************************
*                                                           *
* HTTPParser.java                                           *
* Part of the CDDB-Mp3-Renamer source code licensed under   *
*   the GNU GENERAL PUBLIC LICENSE (GPL).                   *
* Author: Mikael Neman                                      *
* Date: 2003-07-23                                          *
* HTTP: http://cddb-mp3-renamer.flashdance.cx               *
* Description: The parser class of the CDDB page. It will   *
*              download the entire page and search it for   *
*              matching links.                  .           *
*                   .                                       *
************************************************************/

import java.net.*;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.text.*;

import de.ueberdosis.mp3info.*;
import de.ueberdosis.mp3info.id3v2.*;

public class HTTPParser implements Constants
{
  URL cddbUrl;
  byte [] webPage;
  String entirePage;
  private JLabel statusBar;
  private JPanel downSide;
  //Constructor
  HTTPParser(JLabel jl, JPanel p)
  {
    statusBar = jl;
    downSide = p;
  }

  //Downloads a page and returns it as a String
  String downloadPage(String searchString) throws IOException
  {
    if(DEBUG_MODE)
      System.out.println("Entering downloadPage() -" + searchString);
    webPage = new byte[20000];
    //Try to connect
    try
    {
      cddbUrl = new URL(searchString);
    }
    catch (MalformedURLException e) { System.out.println("Caught exception" + e); }

    //Downloading page
    InputStream pageInputStream = cddbUrl.openStream();

    //Read the entire webpage
    int pageSize;
    StringBuffer stringBuffer = new StringBuffer(100000);
    while((pageSize=pageInputStream.read(webPage, 0, 20000)) != -1)
    {
      if(DEBUG_MODE)
        System.out.println("pageSize=" + pageSize);
        String pageString = new String(webPage, 0, pageSize);
        stringBuffer.append(pageString);
    }

    return(stringBuffer.toString());
  }
  int locateLinks(HTMLLinkStorer [] hls, int indexOfPages, String searchString)
  {
    if(DEBUG_MODE)
      System.out.println("Entering locateLinks()");
    int start=0, end=0;
    int page=1;
    int tempIndexStart=0;
    int tempIndexEnd=0;
    //While there still are links on the page
    while(true)
    {
      if(DEBUG_MODE)
        System.out.println("Downloading links from " + page +" to " + (page+9) + " ...");
      statusBar.setText("Downloading links from " + page +" to " + (page+9) + " ...");

      try { entirePage = new String(downloadPage(searchString + "&f=all&s=" + page)); }
      catch (IOException e) { System.out.println("Error: Caught exception in locateLinks in HTTPParser" + e.toString()); }

      start = entirePage.indexOf("LIST START -->");
      end = entirePage.indexOf("LIST END -->");

      //Check if any albums were found, abort if page conations the search string
      if(entirePage.indexOf("We were unable to find a match for the word") != -1)
      {
        if(DEBUG_MODE)
          System.out.println("No albums matches that search criteria");
        statusBar.setText("No albums matches that search criteria");
        return (-1);
      }
      //Check to see if this is the right page, containing some hits
      if(start==-1 || end==-1)
      {
        if(DEBUG_MODE)
        {
          System.out.println("Could not find the required links on the homepage... aborting (this is normal if list is empty)");
          System.out.println("start= " + start + "  end=" + end);
        }

        //check if any hits are found or if there is none on the first page
        if(indexOfPages <= 0)
        {
          //This might not be necessary...
          statusBar.setText("No albums were found on the page...");
        }
        else
          statusBar.setText("Search completed: " + indexOfPages + " albums found");

        break;
      }

      tempIndexStart=start;
      tempIndexEnd=0;

      //Loop through all 10 links (if exist)
      while(true)
      {
        //Find start/end of link and extract it
        //19 is length of search string
        tempIndexStart = (entirePage.indexOf("ENDREL --><A HREF=",tempIndexStart) + 19);
        if(tempIndexStart >= end || tempIndexStart < start)
          break;
        tempIndexEnd = entirePage.indexOf("\"",tempIndexStart);
        String tempAddr = new String(entirePage.substring(tempIndexStart, tempIndexEnd));
        if(DEBUG_MODE)
          System.out.println("got address string: " + tempAddr);

        //Find start/end of name and extract it
        tempIndexStart = (tempIndexEnd + 3);
        tempIndexEnd = (entirePage.indexOf("</A>",tempIndexStart));
        String tempName = new String(entirePage.substring(tempIndexStart, tempIndexEnd));

        //Store information
        hls[indexOfPages] = new HTMLLinkStorer("http://www.cddb.com" + tempAddr,tempName);
        indexOfPages++;
      }
      //Move to next page of links
      page+=10;
    }
    return(indexOfPages);
  }

  AlbumFormatStorer parseAlbum(HTMLLinkStorer hls, ID3Tag [] tags, ID3V2Tag [] v2tags)
  {
    if(DEBUG_MODE)
      System.out.println("Entering parseAlbum()");
    int start=0, end=0;
    int page=1;
    int tempIndexStart=0;
    int tempIndexEnd=0;

    try { entirePage = new String(downloadPage(hls.getAddr())); }
    catch (IOException e) { System.out.println("Caught exception in parseAlbum in HTTPParser" + e.toString()); }

    //start = entirePage.indexOf("Disc Info");
    start = entirePage.indexOf("cddb_returns");
    //end = entirePage.indexOf("Artist Info");
    end = entirePage.indexOf("</table><br>");

    //Check to see if this is the right page, containing some hits
    if(start==-1 || end==-1)
    {
      if(DEBUG_MODE)
      {
        System.out.println("Could not find the required links on the homepage...aborting");
        System.out.println("start= " + start + "  end=" + end);
      }
      //break;
      return(null);
    }

    tempIndexStart=start;
    tempIndexEnd=0;

    //Position 0 is length
    String [] tempReturnStrings1 = new String[100];
    String [] tempReturnStrings2 = new String[100];
    int nrOfSongs=0;

    //It is time to find the artist name
    //Searches for the start
    //First look for disc Info...
    //9 is the length of the string
    int artistStart = (entirePage.indexOf("cddb_returns") +12);
    //...then continue to look for the next "<b>"
    artistStart = ( entirePage.indexOf("<b>", artistStart) +3);
    //Searches for end, starting at the start offset
    int artistEnd = entirePage.indexOf(" /",artistStart);

    if(DEBUG_MODE)
      System.out.println("Parsing artist, start offset: " + artistStart + " end offset " + artistEnd);
    //Abort if error, return null
    if((artistStart < 0 || artistEnd < 0) || (artistStart >= artistEnd))
      return(null);

    String artist = new String(entirePage.substring(artistStart,artistEnd));
    if(DEBUG_MODE)
      System.out.println("Artist name parsed: --> '" + artist + "'");

    //Loop through all XX links (if exist)
    //Artist name has already been found, so now it's time to get the Song names
    while(true)
    {

      //Find start/end of link and extract it
      //Find start of title (4 is length of search string <LI>)
      //tempIndexStart = (entirePage.indexOf("<LI>",tempIndexStart) + 4);
      //tempIndexStart = (entirePage.indexOf("62%>",tempIndexStart) + 4);
      tempIndexStart = (entirePage.indexOf("'62%'>",tempIndexStart) + 6);
      if(DEBUG_MODE)
        System.out.println("start" + start + " end " + end + " indStart " + tempIndexStart + " indEnd " + tempIndexEnd);
      if(tempIndexStart >= end || tempIndexStart < start)
        break;

      tempIndexEnd = entirePage.indexOf("</td>",tempIndexStart);

      //Workaround for embedded artist link in artist name field
      int linkStartIndex = entirePage.indexOf(" -- <a href", tempIndexStart);
      if(linkStartIndex > tempIndexStart && linkStartIndex < tempIndexEnd)
      {
		 tempIndexEnd = linkStartIndex;
	  }

      //Add artist name to the string if it is wanted
      //if(returnFormat == NORMAL_ARTIST_MODE)
      if(DEBUG_MODE)
        System.out.println("Artist name: " + artist);

      //ID3
      tags[nrOfSongs] = new ID3Tag();
      tags[nrOfSongs].setArtist(artist);
      tags[nrOfSongs].setTitle(entirePage.substring(tempIndexStart, tempIndexEnd));
      tags[nrOfSongs].setComment("http://www.flashdance.cx");
      tags[nrOfSongs].setTrack(nrOfSongs+1);

      //ID3v2
      v2tags[nrOfSongs] = new ID3V2Tag();
      FrameTPE1 FrameArtist = new FrameTPE1();
      FrameArtist.setText(artist);
      v2tags[nrOfSongs].addFrame(FrameArtist);
      FrameTIT2 FrameTitle = new FrameTIT2();
      FrameTitle.setText(entirePage.substring(tempIndexStart, tempIndexEnd));
      v2tags[nrOfSongs].addFrame(FrameTitle);
      FrameTRCK FrameTrack = new FrameTRCK();
      Integer trackno = new Integer(nrOfSongs+1);
      FrameTrack.setText(trackno.toString());
      v2tags[nrOfSongs].addFrame(FrameTrack);
      FrameCOMM FrameComment = new FrameCOMM();
      FrameComment.setText("http://www.cddb-mp3-renamer.flashdance.cx/");
      v2tags[nrOfSongs].addFrame(FrameComment);
      //END ID3

      tempReturnStrings1[nrOfSongs++] = new String(artist + " - " + entirePage.substring(tempIndexStart, tempIndexEnd));
      //else if(returnFormat == NO_ARTIST_MODE)
      tempReturnStrings2[nrOfSongs-1] = new String(entirePage.substring(tempIndexStart, tempIndexEnd));

      //This should work...hopefully ;)
      tempIndexStart = (tempIndexEnd );
    }

    AlbumFormatStorer afs = new AlbumFormatStorer();

    String [] returnStrings = new String[nrOfSongs];
    for(int q=0; q < nrOfSongs; q++)
      returnStrings[q] = new String(tempReturnStrings1[q]);
    afs.setNormalArtistModeStrings(returnStrings);

    for(int q=0; q < nrOfSongs; q++)
      returnStrings[q] = new String(tempReturnStrings2[q]);
    afs.setNoArtistModeStrings(returnStrings);

    return(afs);
  }
}
